#include "SearchEvent.hpp"

#include <cerrno>
#include <sys/socket.h>
#include <cstring>
#include <iostream>
#include <ostream>

#include "../../tasks/types.hpp"
#include "../../events/types.hpp"


namespace drp {


void event::SearchEvent::handle(
    Context& context,
    const packet::GenericPacketContent& content,
    const sockaddr_storage& fromAddress,
    const socklen_t fromAddressLength
) {
    packet::GenericPacket packet {};
    packet::GenericPacketContent packetContent {};

    // create the packet header (available to read for everyone)
    packet.channel = 0;
    packet.securityMode = static_cast<std::uint8_t>(packet::SecurityMode::PLAIN);

    // insert our information into the packet
    packetContent.eventType = static_cast<std::uint8_t>(EventType::INFO);
    std::memcpy(&packetContent.data, &context.me, sizeof(context.me));
    packet.setContent(packetContent);

    // TODO(Faraphel): send back the timestamp too

    // send back our information
    if (sendto(
        context.socket,
        &packet,
        sizeof(packet),
        0,
        reinterpret_cast<const sockaddr*>(&fromAddress),
        fromAddressLength
    ) == -1) {
        std::cerr << "[Receiver] Could not send information: " << strerror(errno) << std::endl;
        return;
    }

    char hoststr[NI_MAXHOST];
    char portstr[NI_MAXSERV];
    getnameinfo(
        reinterpret_cast<const sockaddr*>(&fromAddress), fromAddressLength,
        hoststr, sizeof(hoststr),
        portstr, sizeof(portstr),
        NI_NUMERICHOST | NI_NUMERICSERV
    );
    std::cout << "[Receiver] Sent back information to " << hoststr << ":" << portstr << std::endl;
}



}
